/*
 * Copyright (C) 2011 Stefan Schlott
 * Published under the GNU Public License V2 (GPL-2)
 */

import groovy.json.JsonSlurper


def cli = new CliBuilder(usage: 'githubmirror [-g /path/to/git] username destinationdir')
cli.with {
    h longOpt: 'help', 'Show usage information'
    g longOpt: 'git', args: 1, argName: 'gitbin', 'Use this git binary'
}
options = cli.parse(args)

// Print help
if (options.h) {
    cli.usage()
    return 0
}
// Find and verify git binary
def gitbin = options.g
if (!gitbin) {
    // Find git in path
    System.getenv('PATH').split(File.pathSeparator).each { path ->
        def bin = new File("${path}${File.separator}git")
        if (bin.isFile()) {
            gitbin = bin.toString()
        }
    }
}
if (!gitbin) {
    println 'git binary not found'
    return 1
}
if (!new File(gitbin).isFile()) {
    println "Binary ${gitbin} does not exist"
    return 1
}
// Check user name and destination parameter
if (options.arguments().size()!=2) {
    println 'Mandatory parameters: username and destination directory'
    return 1
}

// Create destination directory
def destDir = new File(options.arguments()[1])
if (!destDir.isDirectory()) {
    if (!destDir.mkdirs()) {
        println 'Unable to create destination directory'
        return 1
    }
}

// Get github infos
def repos = []
def userName = options.arguments()[0]
def apiUrl = "http://github.com/api/v2/json/repos/show/${userName}"
while (apiUrl) {
    def url = new URL(apiUrl)
    def conn = url.openConnection()
    conn.connect()
    // Parse json
    def slurper = new JsonSlurper()
    def doc = slurper.parseText(conn.getInputStream().getText())
    repos += doc.repositories
    // Move to next page
    apiUrl = conn.getHeaderField('X-Next')
}

// Create/update mirror
repos.each { repo ->
    def repoDir = new File("${destDir}${File.separator}${repo.name}.git")
    def repoUrl = "git://github.com/${userName}/${repo.name}.git"
    String cmd
    if (!repoDir.isDirectory() || !(new File("${repoDir}/HEAD").isFile())) {
        // Do new clone
        repoDir.mkdir()
        cmd = "${gitbin} clone --mirror ${repoUrl} ."
    } else {
        // Update
        cmd = "${gitbin} fetch"
    }
    def sout = new StringBuffer(), serr = new StringBuffer()
    def proc = cmd.execute(null, repoDir)
    proc.consumeProcessOutput(sout, serr)
    def result = proc.waitFor()
    if (result!=0) {
        println "Error running command: ${cmd}"
        println serr
    }
}
